require 'English'

require 'json'
require 'tmpdir'

lib = File.expand_path(__dir__)
$LOAD_PATH.unshift(lib) unless $LOAD_PATH.include?(lib)

require "TrRMIte/version"

# TODO replace with Rinda ??
# https://ruby-doc.org/stdlib-2.1.1/libdoc/rinda/rdoc

module TrRMIte

    DEFAULT_DIRECTORY = '/var/lib/trrmite'

    if RUBY_PLATFORM =~ /darwin/i
      require 'mkmf'

      MakeMakefile::Logging.quiet = true
      MakeMakefile::Logging.logfile(File::NULL)

      Dir.chdir(Dir.tmpdir) do
        if MakeMakefile.find_executable('brew')
          DEFAULT_DIRECTORY.prepend %x(brew --prefix).chomp
        end
      end
    end

    DEFAULT_TOPIC        = 'test-messages' # see `ruby-kafka` (~ firehose examples)

    DEFAULT_REGISTRY_URI = ENV.fetch('TRRMITE_REGISTRY_URI', 'druby://127.0.0.1:9999')
    DEFAULT_QUEUE_URI    = ENV.fetch('TRRMITE_QUEUE_URI',    'druby://127.0.0.1:1111')

    INTERNAL_TOPIC     = (ENV['TRRMITE_INTERNAL_TOPIC'] || 'TrRMIte').freeze
    SCHEDULER_INTERVAL = (ENV['TRRMITE_SCHEDULER_INTERVAL'] || 60).to_i

    DEFAULT_OFFSET_STORE    = File.join(DEFAULT_DIRECTORY, 'trrmite.offset_store')
    DEFAULT_EVENT_LOG       = File.join(DEFAULT_DIRECTORY, 'trrmite.event_log')
    DEFAULT_SCHEDULER_STATE = File.join(DEFAULT_DIRECTORY, 'trrmite.scheduler_state')
    DEFAULT_BACKUPS_DIR     = File.join(DEFAULT_DIRECTORY, 'trrmite.backups')

    class OutOfSequence < StandardError ; end
    class WrongTopic    < StandardError ; end

    def wall_clock_time(humanised = false)
      # always in "seconds since epoch"
      # (and obviously always in "UTC")
      humanised ? Time.now.utc.to_s : Time.now.utc.to_f
    end
    module_function :wall_clock_time

    def publish(*args)
      @producer ||= Producer.new(DEFAULT_QUEUE_URI)
      if args.count == 1
        @producer.publish(args.first)
      else
        topic, type, payload = *args
        @producer.publish(Event.new(topic: topic, type: type, payload: payload))
      end
    end
    module_function :publish

    def schedule(*args)
      @producer ||= Producer.new(DEFAULT_QUEUE_URI)
      if args.count == 2
        @producer.schedule(args.first, args.last)
      else
        topic, type, payload, publish_at = *args
        @producer.schedule(Event.new(topic: topic, type: type, payload: payload), publish_at)
      end
    end
    module_function :schedule

    def inflect(name, *inflectors)
      inflectors.unshift :downcase # by default, `downcase` is the only one!
      inflectors.inject(name.gsub(/(.)([A-Z])(?![A-Z])/, '\1_\2')) { |word, inflector|
        word.send(inflector)
      }
    end
    module_function :inflect

end

require 'ext/argv'
require 'ext/string'
require 'ext/commify'
require 'ext/numeric'

require 'drb/drb'

require 'TrRMIte/event'
require 'TrRMIte/broker'

require 'TrRMIte/event_handling'

require 'TrRMIte/defaults'

require 'TrRMIte/consumer'
require 'TrRMIte/producer'

require 'TrRMIte/consumers'
require 'TrRMIte/producers'
