require 'securerandom'

module PayloadLookup

  def [](key)
    super
  rescue NameError
    payload[key]
  end

end

module TimestampResolution

  def timestamp_in(resolution = nil)
    case resolution
    when nil, :seconds then timestamp
    when :milliseconds then (timestamp * 1_000).to_i
    else
      raise "Invalid timestamp resolution: #{resolution}"
    end
  end

end

::TrRMIte::Event = Struct.new(:uuid, :topic, :type, :payload, :sequence, :tsequence, :timestamp, :causation_id, :correlation_id) do

  def initialize(event_hash = {})
    event_hash[:uuid] ||= SecureRandom.uuid
    super(
      event_hash[:uuid],
      event_hash[:topic],
      event_hash[:type],
      event_hash[:payload],
      event_hash[:sequence],
      event_hash[:tsequence],
      event_hash[:timestamp],
      event_hash[:causation_id],
      event_hash[:correlation_id],
    )
  end

  def to_json
    to_h.to_json
  end

  include PayloadLookup
  include TimestampResolution

  def self.parse(event_json, additions = {})
    self.from(JSON.parse(event_json, symbolize_names: true), additions)
  end

  def self.from(event_hash, additions = {})
    self.new(event_hash.merge(additions))
  end

end
