module TrRMIte
  module EventHandling
    module DSL

      def self.included(base)
        class << base
          attr_reader :event_handlers
        end
        base.extend(ClassMethods)
      end

      module ClassMethods
        def handle(*event_types, &handler)
          raise ArgumentError, "missing event handler" unless handler
          @event_handlers ||= begin
                                handlers = {}
                                def handlers.for(event)
                                  self[TrRMIte.inflect(event.type, :to_sym)]
                                end
                                handlers
                              end

          event_types.each do |event_type|
            @event_handlers[event_type] = handler
          end
        end

        def handler_for(event)
          @event_handlers ? @event_handlers.for(event) : nil
        end

        def handles?(event)
          !handler_for(event).nil?
        end
      end

      def handler_for(event)
        self.class.handler_for(event)
      end

      def handles?(event)
        self.class.handles?(event)
      end
    end

    module Processing

      def self.included(base)
        base.prepend(ConditionalPersisting)
      end

      def process(event)
        if handler = handler_for(event)
          instance_exec(event, &handler)
        else
          # subclasses that don't register explicit, type-specific event handlers
          # can always process all topic events by overriding `process(event)`...
          # raise 'SubclassResponsibility'
        end
      end

      module ConditionalPersisting
        # prepending this module ensures that consumers
        # that include the EventHandling DSL will receive
        # the `persist_state` callback **if and only if**
        # they handled the event thus avoiding superfluous
        # callbacks and possibly expensive save operations
        def persist_state(event)
          super if handles? event
        end
      end

    end
  end
end
