module TrRMIte
  class Producer

    def initialize(queue_uri = nil)
      DRb.start_service # ensure DRb is running for this producer process
      queue_uri ||= TrRMIte::DEFAULT_QUEUE_URI
      @queue = DRbObject.new_with_uri(queue_uri)
    end

    def publish event
      @queue.push event.to_json
    end

    def schedule event, publish_at = 0
      publish(
        Event.new(
          topic:   TrRMIte::INTERNAL_TOPIC,
          type:    'FutureEventScheduled',
          payload: {
            scheduled_event: event.to_h,
            scheduled_at:    TrRMIte.wall_clock_time,
            publish_at:      publish_at,
          },
        )
      )
    end

    def publish_scheduled event
      scheduled_event = Event.from(event[:scheduled_event])
      publish(scheduled_event)

      publish(
        Event.new(
          topic:   TrRMIte::INTERNAL_TOPIC,
          type:    'ScheduledEventPublished',
          payload: {
            published_at: TrRMIte.wall_clock_time,
          },
          causation_id:   event.uuid,
          correlation_id: scheduled_event[:uuid],
        )
      )
    end

  end

  class EventProducer < Producer

    attr_reader :events

    def initialize(queue_uri)
      super
      @events = Enumerator.new do |yielder|
        loop do
          if (event = next_event)
            yielder << event
          else
            raise StopIteration
          end
        end
      end
    end

    private

    def next_event
      raise 'SubclassResponsibility'
    end

  end

  class ProxyProducer < EventProducer

    attr_reader :queue

    def initialize(queue_uri)
      super
      @queue = Queue.new
    end

    private

    def next_event
      @queue.pop
    end

  end
end
